CON

  _clkmode      = xtal1 + pll16x
  _xinfreq      = 5_000_000

  rf_pin        = 3
  fb_pin        = 4
  if_pin        = 8
  qf_pin        = 9
  au_pin        = 10
  fsample       = 16000
  clks          = 80_000_000 / fsample
  mclks         = clks / 2

OBJ

  u : "umath"
  tv : "tv_wtext"

VAR

  long  qStack[32]
  long  dStack[32]
  long  xStack[32]
  long  audio, frequency, sum, avg, dif, n
               
PUB Start | tune_freq

  tv.start(12)
  dira[fb_pin]~~
  ctra := %01001 << 26 | fb_pin << 9 | rf_pin
  frqa := 1
  q_freq := @frequency
  d_audio := a_audio := @audio
  cognew(@quadrature, 0)
  cognew(@demod, 0)
  cognew(@audio_out, 0)  
  repeat
    repeat tune_freq from 550_000 to 1600_000 step 10_000
      frequency := u.div(tune_freq, 0, clkfreq)
      tv.out(0)
      tv.decn(tune_freq / 1000, 4)
      tv.out(".")
      tv.decn(tune_freq // 1000, -3)
      tv.str(string(" KHz "))
      waitcnt(cnt + clkfreq * 3)

DAT

'' Generate two square waves, 90 degrees out of phase.

              org       0
quadrature    mov       dira,q_dira0
              mov       ctra,q_ctra0
              mov       ctrb,q_ctrb0
              
:query        rdlong    freq,q_freq
              cmp       freq,frqa wz
        if_z  jmp       #:query

              mov       frqa,#0
              mov       frqb,#0
              mov       phsa,#0
              mov       phsb,_4000_0000
              add       phsb,freq
              add       phsb,freq
              add       phsb,freq
              add       phsb,freq
              mov       frqa,freq
              mov       frqb,freq
              jmp       #:query

q_freq        long      0-0
q_ctra0       long      %00100 << 26 | if_pin
q_ctrb0       long      %00100 << 26 | qf_pin
q_dira0       long      1 << if_pin | 1 << qf_pin

_4000_0000    long      $4000_0000

freq          res       1

''===================================================================

'' I and Q mixers and detector.

              org       0
demod         mov       frqa,#1
              mov       frqb,#1
              mov       ctra,d_ctra0
              mov       ctrb,d_ctrb0
              mov       time,cnt
              add       time,sample

:mix          waitcnt   time,sample
              mov       i_samp,phsa
              mov       phsa,#0
              mov       q_samp,phsb
              mov       phsb,#0
              mov       acc,i_samp
              call      #square
              mov       i_samp,acc
              mov       acc,q_samp
              call      #square
              add       acc,i_samp
              call      #sqrt
              wrlong    acc,d_audio
              jmp       #:mix
              
''--------[square]---------------------------------------------------

'' acc = (acc - half) ** 2   (Multiply routine from "Propeller Guts".)

square        sub       acc,half
              abs       acc,acc
              mov       ctr,#16
              mov       accx,acc
              shl       accx,#16
              shr       acc,#1 wc
              
:sqlp   if_c  add       acc,accx wc
              rcr       acc,#1 wc
              djnz      ctr,#:sqlp

square_ret    ret

''--------[sqrt]-----------------------------------------------------

'' acc = sqrt(acc)      (Square root routine from "Propeller Guts".)

sqrt          mov       accy,#0
              mov       accx,#0
              mov       ctr,#16
:rtlp         shl       acc,#1 wc

              rcl       accy,#1
              shl       acc,#1 wc
              rcl       accy,#1
              shl       accx,#2 
              or        accx,#1
              cmpsub    accy,accx wc
              shr       accx,#2
              rcl       accx,#1
              djnz      ctr,#:rtlp

              mov       acc,accx
sqrt_ret      ret

''-------------------------------------------------------------------

d_audio       long      0-0
d_ctra0       long      %10110 << 26 | if_pin << 9 | fb_pin
d_ctrb0       long      %10110 << 26 | qf_pin << 9 | fb_pin
sample        long      clks
half          long      mclks

acc           res       1
accx          res       1
accy          res       1
ctr           res       1
i_samp        res       1
q_samp        res       1
time          res       1

''===================================================================

'' Audio Output

              org       0
audio_out     mov       ctra,a_ctra0
              mov       dira,a_dira0

:aulp         rdlong    aud,a_audio
              shl       aud,#20
              mov       frqa,aud
              jmp       #:aulp

a_audio       long      0-0
a_ctra0       long      %00110 << 26 | au_pin
a_dira0       long      1 << au_pin

aud           res       1



  